/* ISC license. */

/* Generated by: ./configure --prefix=/usr --sysconfdir=/etc --disable-allstatic --disable-static --enable-pkgconfig --enable-shared --enable-ssl=bearssl --with-pkgconfig */

#ifndef S6_NETWORKING_CONFIG_H
#define S6_NETWORKING_CONFIG_H

#define S6_NETWORKING_VERSION "2.8.0.0"
#define S6_NETWORKING_BINPREFIX ""
#define S6_NETWORKING_EXTBINPREFIX ""
#define S6_NETWORKING_EXTLIBEXECPREFIX "/usr/libexec/"
#define S6_NETWORKING_LIBEXECPREFIX "/usr/libexec/"
#define S6_NETWORKING_SYSCONFPREFIX "/etc/"
#undef S6_NETWORKING_USE_EXECLINE
#define S6_NETWORKING_USE_EXECLINE
#undef S6_NETWORKING_USE_TLS
#undef S6_NETWORKING_USE_BEARSSL
#define S6_NETWORKING_USE_BEARSSL

#endif
